import wx
import string
import os

import shutil

try:
    from agw import customtreectrl as CT
except ImportError:  # if it's not there locally, try the wxPython lib.
    import wx.lib.agw.customtreectrl as CT

ArtIDs = ["ADD_BOOKMARK",
          "DEL_BOOKMARK",
          "HELP_SIDE_PANEL",
          "HELP_SETTINGS",
          "HELP_BOOK",
          "HELP_FOLDER",
          "HELP_PAGE",
          "GO_BACK",
          "GO_FORWARD",
          "GO_UP",
          "GO_DOWN",
          "GO_TO_PARENT",
          "GO_HOME",
          "FILE_OPEN",
          "PRINT",
          "HELP",
          "TIP",
          "REPORT_VIEW",
          "LIST_VIEW",
          "NEW_DIR",
          "HARDDISK",
          "FLOPPY",
          "CDROM",
          "REMOVABLE",
          "FOLDER",
          "FOLDER_OPEN",
          "GO_DIR_UP",
          "EXECUTABLE_FILE",
          "NORMAL_FILE",
          "TICK_MARK",
          "CROSS_MARK",
          "ERROR",
          "QUESTION",
          "WARNING",
          "INFORMATION",
          "MISSING_IMAGE",
          ]

Event = None


class FileList(CT.CustomTreeCtrl):

    def __init__(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition,
                 size=wx.DefaultSize,
                 style=wx.SUNKEN_BORDER | wx.WANTS_CHARS,
                 agwStyle=CT.TR_HAS_BUTTONS | CT.TR_HAS_VARIABLE_ROW_HEIGHT,
                 log=None, rootPath=r"..\testscripts", eventBus=None, EVENT=None):
        CT.CustomTreeCtrl.__init__(self, parent, id, pos, size, style, agwStyle)

        self.SetBackgroundColour(wx.WHITE)
        self.eventBus = eventBus
        global Event
        Event = EVENT
        self.checkedItemPath = {}

        alldata = dir(CT)
        treestyles = []
        events = []
        for data in alldata:
            if data.startswith("TR_"):
                treestyles.append(data)
            elif data.startswith("EVT_"):
                events.append(data)
        self.events = events
        self.styles = treestyles
        self.item = None
        self.windowed_item = None

        il = wx.ImageList(16, 16)

        for items in ArtIDs:
            il.Add(wx.ArtProvider.GetBitmap(eval("wx.ART_{}".format(items)), wx.ART_TOOLBAR,
                                            (16, 16)))

        self.AssignImageList(il)
        self.count = 0
        self.log = log

        self.rootPath = os.path.abspath(rootPath)
        self.root = self.AddRoot(self.rootPath)

        self.SetPyData(self.root, None)
        self.SetItemImage(self.root, 24, CT.TreeItemIcon_Normal)
        self.SetItemImage(self.root, 13, CT.TreeItemIcon_Expanded)

        self.updateChild(self.root, self.rootPath)

        self.Bind(wx.EVT_LEFT_DCLICK, self.OnLeftDClick)
        self.Bind(wx.EVT_IDLE, self.OnIdle)

        self.eventdict = {'BEGIN_DRAG': self.OnBeginDrag,
                          'BEGIN_LABEL_EDIT': self.OnBeginEdit,
                          'BEGIN_RDRAG': self.OnBeginRDrag,
                          'DELETE_ITEM': self.OnDeleteItem,
                          'END_DRAG': self.OnEndDrag,
                          'END_LABEL_EDIT': self.OnEndEdit,
                          'ITEM_ACTIVATED': self.OnActivate,
                          'ITEM_CHECKED': self.OnItemCheck,
                          'ITEM_CHECKING': self.OnItemChecking,
                          'ITEM_COLLAPSED': self.OnItemCollapsed,
                          'ITEM_COLLAPSING': self.OnItemCollapsing,
                          'ITEM_EXPANDED': self.OnItemExpanded,
                          'ITEM_EXPANDING': self.OnItemExpanding,
                          'ITEM_GETTOOLTIP': self.OnToolTip,
                          'ITEM_MENU': self.OnItemMenu,
                          'ITEM_RIGHT_CLICK': self.OnRightDown,
                          'KEY_DOWN': self.OnKey,
                          'SEL_CHANGED': self.OnSelChanged,
                          'SEL_CHANGING': self.OnSelChanging,
                          "ITEM_HYPERLINK": self.OnHyperLink}

        mainframe = wx.GetTopLevelParent(self)

        evt_2_func = {CT.EVT_TREE_ITEM_EXPANDED: self.OnItemExpanded,
                      CT.EVT_TREE_ITEM_EXPANDING: self.OnItemExpanding,
                      CT.EVT_TREE_ITEM_COLLAPSED: self.OnItemCollapsed,
                      CT.EVT_TREE_ITEM_COLLAPSING: self.OnItemCollapsing,
                      CT.EVT_TREE_SEL_CHANGED: self.OnSelChanged,
                      CT.EVT_TREE_SEL_CHANGING: self.OnSelChanging,
                      wx.EVT_RIGHT_DOWN: self.OnRightDown,
                      wx.EVT_RIGHT_UP: self.OnRightUp}

        if not hasattr(mainframe, "leftpanel"):
            for _evt, _func in evt_2_func.items():
                self.Bind(_evt, _func)

        else:
            for combos in mainframe.treeevents:
                self.binding_events(combos)

        if hasattr(mainframe, "leftpanel"):
            self.ChangeStyle(mainframe.treestyles)

        self.SelectItem(self.root)
        self.ExpandAll()

    def refreshTree(self, rootPath: str = None, filterValue: str = None):
        self.Freeze()
        self.DeleteAllItems()

        if rootPath and len(rootPath) > 0:
            self.rootPath = os.path.abspath(rootPath)
        self.root = self.AddRoot(self.rootPath)
        self.SetPyData(self.root, None)
        self.SetItemImage(self.root, 24, CT.TreeItemIcon_Normal)
        self.SetItemImage(self.root, 13, CT.TreeItemIcon_Expanded)

        self.updateChild(self.root, self.rootPath, filterValue)
        self.Thaw()
        self.ExpandAll()
        pass

    def ChangeStyle(self, combos):

        style = 0
        for combo in combos:
            if combo.GetValue() == 1:
                style = style | eval("CT." + combo.GetLabel())

        if self.GetAGWWindowStyleFlag() != style:
            self.SetAGWWindowStyleFlag(style)

    def updateChild(self, parent, parentPath, filterValue: str = None):
        for fileName in os.listdir(parentPath):
            abspath = os.path.join(parentPath, fileName)
            isdir = os.path.isdir(abspath)

            if isdir:
                if not fileName.startswith("."):
                    dirItem = self.AppendItem(parent, fileName, ct_type=1)
                    self.SetPyData(dirItem, None)
                    self.SetItemImage(dirItem, 24, CT.TreeItemIcon_Normal)
                    self.SetItemImage(dirItem, 13, CT.TreeItemIcon_Expanded)
                    self.updateChild(dirItem, abspath, filterValue)
                    if filterValue and len(filterValue) > 0:
                        if not self.ItemHasChildren(dirItem):
                            self.Delete(dirItem)
            else:
                if filterValue and len(filterValue) > 0:
                    if fileName.lower().find(filterValue.lower()) == -1:
                        continue
                if fileName.endswith(".py") or fileName.endswith(".ini") or fileName.endswith(".txt"):
                    fileItem = self.AppendItem(parent, fileName, ct_type=1)
                    self.SetPyData(fileItem, None)
                    self.SetItemImage(fileItem, 28, CT.TreeItemIcon_Normal)
        pass

    def getFullPath(self, item):
        fullPath = []
        while item:
            fullPath.append(self.GetItemText(item))
            item = item.GetParent()
        fullPath.reverse()
        return "\\".join(fullPath)

    def OnCompareItems(self, item1, item2):
        if self.GetItemText(item1) < self.GetItemText(item2):
            return -1
        elif self.GetItemText(item1) > self.GetItemText(item2):
            return 1
        else:
            return 0

    def binding_events(self, choice, recreate=False):

        value = choice.GetValue()
        text = choice.GetLabel()
        evt = "CT." + text
        binder = self.eventdict[text[9:]]
        if value != 1:
            self.Bind(eval(evt), None)
            if evt == "CT.EVT_TREE_BEGIN_RDRAG":
                self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)
                self.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)
        else:
            if evt == "CT.EVT_TREE_BEGIN_RDRAG":
                self.Bind(wx.EVT_RIGHT_DOWN, None)
                self.Bind(wx.EVT_RIGHT_UP, None)
            self.Bind(eval(evt), binder)

    def OnIdle(self, event):
        event.Skip()

    def OnRightDown(self, event):

        pt = event.GetPosition()
        item, flags = self.HitTest(pt)

        self.UpdateAllItemChecked()

        # for _path in self.checkedItemPath:
        # logger.debug("checkedItemPath= {}".format(_path))

        if item:
            self.item = item
            self.SelectItem(item)

    def OnRightUp(self, event):

        item = self.item

        if not item:
            event.Skip()
            return

        self.current = item
        _itemText = self.GetItemText(item)

        if _itemText.endswith(".py"):
            menu = wx.Menu()
            item_rename = menu.Append(wx.ID_ANY, "Rename...")

            menu.AppendSeparator()

            # tests script run/stop/pause
            item_run = menu.Append(wx.ID_ANY, 'Run "{}"'.format(self.GetItemText(self.current)))

            item_stop = menu.Append(wx.ID_ANY, "Stop")
            menu.AppendSeparator()

            if len(self.checkedItemPath) > 0:
                item_run_select = menu.Append(wx.ID_ANY,
                                              'Run selected files')
                item_save_select = menu.Append(wx.ID_ANY,
                                               'Save selected files')

                self.Bind(wx.EVT_MENU, self.OnItemRunSelect, item_run_select)
                self.Bind(wx.EVT_MENU, self.OnItemSaveSelect, item_save_select)
            menu.AppendSeparator()

            item_delete = menu.Append(wx.ID_ANY, "Delete...")

            if item == self.GetRootItem():
                item_delete.Enable(False)

            self.Bind(wx.EVT_MENU, self.OnItemRun, item_run)
            self.Bind(wx.EVT_MENU, self.OnItemStop, item_stop)
            self.Bind(wx.EVT_MENU, self.OnItemRename, item_rename)
            self.Bind(wx.EVT_MENU, self.OnItemDelete, item_delete)

            self.PopupMenu(menu)
            menu.Destroy()
            return

        elif _itemText.endswith(".txt"):
            menu = wx.Menu()
            # menu.AppendSeparator()

            if len(self.checkedItemPath) > 0:
                item_run_select = menu.Append(wx.ID_ANY,
                                              'Run selected files')
                item_save_select = menu.Append(wx.ID_ANY,
                                               'Save selected files')

                self.Bind(wx.EVT_MENU, self.OnItemRunSelect, item_run_select)
                self.Bind(wx.EVT_MENU, self.OnItemSaveSelect, item_save_select)
            menu.AppendSeparator()

            item_rename = menu.Append(wx.ID_ANY, "Rename...")
            item_delete = menu.Append(wx.ID_ANY, "Delete...")

            if item == self.GetRootItem():
                item_delete.Enable(False)

            self.Bind(wx.EVT_MENU, self.OnItemRename, item_rename)
            self.Bind(wx.EVT_MENU, self.OnItemDelete, item_delete)

            self.PopupMenu(menu)
            menu.Destroy()
            return

        elif _itemText.endswith(".ini"):
            menu = wx.Menu()
            # menu.AppendSeparator()

            # tests script run/stop/pause
            item_run = menu.Append(wx.ID_ANY, 'Run "{}"'.format(self.GetItemText(self.current)))

            item_stop = menu.Append(wx.ID_ANY, "Stop")
            menu.AppendSeparator()

            if len(self.checkedItemPath) > 0:
                item_run_select = menu.Append(wx.ID_ANY,
                                              'Run selected files')
                item_save_select = menu.Append(wx.ID_ANY,
                                               'Save selected files')

                self.Bind(wx.EVT_MENU, self.OnItemRunSelect, item_run_select)
                self.Bind(wx.EVT_MENU, self.OnItemSaveSelect, item_save_select)
            menu.AppendSeparator()

            item_rename = menu.Append(wx.ID_ANY, "Rename...")
            item_delete = menu.Append(wx.ID_ANY, "Delete...")

            if item == self.GetRootItem():
                item_delete.Enable(False)

            self.Bind(wx.EVT_MENU, self.OnItemRun, item_run)
            self.Bind(wx.EVT_MENU, self.OnItemStop, item_stop)
            self.Bind(wx.EVT_MENU, self.OnItemRename, item_rename)
            self.Bind(wx.EVT_MENU, self.OnItemDelete, item_delete)

            self.PopupMenu(menu)
            menu.Destroy()
            return

        else:
            menu = wx.Menu()
            item_create_folder = menu.Append(wx.ID_ANY, "new folder")
            item_create = menu.Append(wx.ID_ANY, "new file (Empty)")
            item_create_with_temp = menu.Append(wx.ID_ANY, "new file (With template)")

            menu.AppendSeparator()

            if len(self.checkedItemPath) > 0:
                item_run_select = menu.Append(wx.ID_ANY,
                                              'Run selected files')
                item_save_select = menu.Append(wx.ID_ANY,
                                               'Save selected files')

                self.Bind(wx.EVT_MENU, self.OnItemRunSelect, item_run_select)
                self.Bind(wx.EVT_MENU, self.OnItemSaveSelect, item_save_select)
            menu.AppendSeparator()

            item_rename = menu.Append(wx.ID_ANY, "Rename...")
            item_delete = menu.Append(wx.ID_ANY, "Delete...")

            if item == self.GetRootItem():
                item_delete.Enable(False)

            self.Bind(wx.EVT_MENU, self.OnItemRename, item_rename)
            self.Bind(wx.EVT_MENU, self.OnItemDelete, item_delete)
            self.Bind(wx.EVT_MENU, self.OnItemCreateFile, item_create)
            self.Bind(wx.EVT_MENU, self.OnItemCreateFolder, item_create_folder)
            self.Bind(wx.EVT_MENU, self.OnItemCreateFileWithTemp, item_create_with_temp)

            self.PopupMenu(menu)
            menu.Destroy()
            return

    def OnItemDelete(self, event):

        strs = "Are You Sure You Want To Delete Item " + self.GetItemText(self.current) + "?"
        dlg = wx.MessageDialog(None, strs, 'Deleting Item',
                               wx.YES_NO | wx.NO_DEFAULT | wx.ICON_QUESTION)

        if dlg.ShowModal() == wx.ID_NO:
            dlg.Destroy()
            return

        dlg.Destroy()
        self.DeleteChildren(self.current)
        self.Delete(self.current)
        self.DeleteFile(self.getFullPath(self.current))
        self.current = None

    def OnItemCreateFile(self, event):

        dlg = wx.TextEntryDialog(self, "Please Enter The New Item Name", 'Item Naming', 'newScript')
        if dlg.ShowModal() == wx.ID_OK:
            newname = dlg.GetValue()
            newFilePath = self.getFullPath(self.current) + "\\{}".format(newname)
            if not os.path.exists(newFilePath):
                with open(newFilePath, "w") as newFile:
                    pass
            newitem = self.AppendItem(self.current, newname, ct_type=1)
            self.SetPyData(newitem, None)
            self.SetItemImage(newitem, 28, CT.TreeItemIcon_Normal)
            self.EnsureVisible(newitem)

        dlg.Destroy()

    def OnItemCreateFileWithTemp(self, event):

        dlg = wx.TextEntryDialog(self, "Please Enter The New Item Name", 'Item Naming',
                                 'newScript')

        if dlg.ShowModal() == wx.ID_OK:
            newname = dlg.GetValue()
            newFilePath = self.getFullPath(self.current) + "\\{}".format(newname)
            # TODO should we need to set temp file path in the config panel?
            tempFilePath = self.rootPath + "\\TEMPLATES\\template.py"
            if os.path.exists(newFilePath):
                self.eventBus.post(Event.ShowMessageDlg(msg="This file already exists."))
            elif not os.path.exists(tempFilePath):
                self.eventBus.post(Event.ShowMessageDlg(msg="The temp file does not exist in {}.".
                                                        format(tempFilePath)))
            else:
                shutil.copy(tempFilePath, newFilePath)
                newItem = self.AppendItem(self.current, newname, ct_type=1)
                self.SetPyData(newItem, None)
                self.SetItemImage(newItem, 28, CT.TreeItemIcon_Normal)
                self.EnsureVisible(newItem)

        dlg.Destroy()

    def OnSeparatorInsert(self, event):

        newitem = self.InsertSeparator(self.GetItemParent(self.current), self.current)
        self.EnsureVisible(newitem)

    def OnBeginEdit(self, event):

        # show how to prevent edit...
        item = event.GetItem()
        if item and self.GetItemText(item) == "The Root Item":
            wx.Bell()

            # Lets just see what's visible of its children
            cookie = 0
            root = event.GetItem()
            (child, cookie) = self.GetFirstChild(root)

            while child:
                (child, cookie) = self.GetNextChild(root, cookie)

            event.Veto()

    def OnEndEdit(self, event):

        # show how to reject edit, we'll not allow any digits
        for x in event.GetLabel():
            if x in string.digits:
                event.Veto()
                return

    def OnLeftDClick(self, event):
        pt = event.GetPosition()
        item, flags = self.HitTest(pt)
        if os.path.isfile(self.getFullPath(item)):
            self.eventBus.post(Event.OpenScriptFile(self.GetItemText(item), self.getFullPath(item)))
        if item and (flags & CT.TREE_HITTEST_ONITEMLABEL):
            if self.GetAGWWindowStyleFlag() & CT.TR_EDIT_LABELS:
                self.EditLabel(item)
        event.Skip()

    def OnItemExpanded(self, event):

        item = event.GetItem()
        if item:
            pass

    def OnItemExpanding(self, event):

        item = event.GetItem()
        if item:
            if item == self.windowed_item:
                item.DeleteWindow()
            child = self.GetLastChild(item)
        event.Skip()

    def OnItemCollapsed(self, event):

        item = event.GetItem()
        if item:
            pass

    def OnItemCollapsing(self, event):

        item = event.GetItem()
        if item:
            if item == self.windowed_item:
                item.DeleteWindow()
                pass
        event.Skip()

    def OnSelChanged(self, event):

        self.item = event.GetItem()
        event.Skip()

    def OnSelChanging(self, event):

        item = event.GetItem()
        olditem = event.GetOldItem()

        if item:
            if not olditem:
                olditemtext = "None"
            else:
                olditemtext = self.GetItemText(olditem)

        event.Skip()

    def OnBeginDrag(self, event):

        self.item = event.GetItem()
        if self.item:
            event.Allow()

    def OnBeginRDrag(self, event):

        self.item = event.GetItem()
        if self.item:
            event.Allow()

    def OnEndDrag(self, event):

        self.item = event.GetItem()
        event.Skip()

    def OnDeleteItem(self, event):

        item = event.GetItem()

        if not item:
            return
        event.Skip()

    def OnItemCheck(self, event):
        item = event.GetItem()
        children = item.GetChildren()
        event.Skip()

    def UpdateAllItemChecked(self, parent: CT.GenericTreeItem = None):
        if not parent:
            self.checkedItemPath.clear()
            parent = self.root

        parent.IsChecked()
        for _child in parent.GetChildren():
            if not _child.HasChildren():
                full_path = self.getFullPath(_child)
                if os.path.isfile(full_path) and \
                        _child.IsChecked() and full_path not in self.checkedItemPath:
                    self.checkedItemPath[full_path] = self.GetItemText(_child)
            else:
                self.UpdateAllItemChecked(_child)

    def OnItemChecking(self, event):

        item = event.GetItem()
        event.Skip()

    def OnToolTip(self, event):

        item = event.GetItem()
        if item:
            event.SetToolTip(wx.ToolTip(self.GetItemText(item)))

    def OnItemMenu(self, event):

        item = event.GetItem()

        event.Skip()

    def OnKey(self, event):

        keycode = event.GetKeyCode()
        keyname = eval("wx.{}".format(keycode))

        if keycode == wx.WXK_BACK:
            # self.log.write("OnKeyDown: HAHAHAHA! I Vetoed Your Backspace! HAHAHAHA\n")
            return

        if keyname is None:
            if "unicode" in wx.PlatformInfo:
                keycode = event.GetUnicodeKey()
                if keycode <= 127:
                    keycode = event.GetKeyCode()
                keyname = "\"" + chr(event.GetUnicodeKey()) + "\""
                if keycode < 27:
                    keyname = "Ctrl-%s" % chr(ord('A') + keycode - 1)

            elif keycode < 256:
                if keycode == 0:
                    keyname = "NUL"
                elif keycode < 27:
                    keyname = "Ctrl-%s" % chr(ord('A') + keycode - 1)
                else:
                    keyname = "\"%s\"" % chr(keycode)
            else:
                keyname = "unknown (%s)" % keycode

        event.Skip()

    def OnActivate(self, event):

        event.Skip()

    def OnHyperLink(self, event):
        item = event.GetItem()

    def OnTextCtrl(self, event):

        keycode = event.GetKeyCode()
        char = chr(keycode) if keycode < 256 else ''
        event.Skip()

    def OnComboBox(self, event):
        selection = event.GetEventObject().GetValue()
        event.Skip()

    def OnItemRun(self, event):
        if self.current is not None and not self.current.GetChildren():
            self.eventBus.post(Event.RunSingleScript(self.GetItemText(self.current),
                                                     self.getFullPath(self.current)))
        pass

    def OnItemRunSelect(self, event):
        self.eventBus.post(Event.RunMultipleScript(self.checkedItemPath))
        pass

    def OnItemSaveSelect(self, event):
        self.eventBus.post(Event.SaveMultipleScript(self.checkedItemPath))
        pass

    def OnItemStop(self, event):
        self.eventBus.post("", tag=Event.System.STOP.TAG)
        pass

    def OnItemPause(self, event):
        pass

    def DeleteFile(self, filePath: str = None):
        if os.path.isfile(filePath):
            os.remove(filePath)
        elif os.path.isdir(filePath):
            shutil.rmtree(filePath)
        pass

    def OnItemRename(self, event):
        dlg = wx.TextEntryDialog(self, "Please Enter The New Name", 'Renaming',
                                 self.GetItemText(self.current))

        if dlg.ShowModal() == wx.ID_OK:
            newname = dlg.GetValue()
            shutil.move(self.getFullPath(self.current),
                        self.getFullPath(self.current.GetParent()) + "\\{}".format(newname))
            self.current.SetText(newname)

        dlg.Destroy()
        pass

    def OnItemCreateFolder(self, event):

        dlg = wx.TextEntryDialog(self, "Please Enter The folder Name", 'Item Naming', 'newFolder')
        if dlg.ShowModal() == wx.ID_OK:
            newname = dlg.GetValue()
            newFilePath = self.getFullPath(self.current) + "\\{}".format(newname)
            if not os.path.exists(newFilePath):
                os.mkdir(newFilePath)
            dirItem = self.AppendItem(self.current, newname, ct_type=1)
            self.SetPyData(dirItem, None)
            self.SetItemImage(dirItem, 24, CT.TreeItemIcon_Normal)
            self.SetItemImage(dirItem, 13, CT.TreeItemIcon_Expanded)
        dlg.Destroy()



import datetime
from xml.sax import saxutils
import sys

# ----------------------------------------------------------------------
# Template

class Template_HTML_Report(object):
    """
    Define a HTML template for report customerization and generation.

    Overall structure of an HTML report

    HTML
    +------------------------+
    |<html>                  |
    |  <head>                |
    |                        |
    |   STYLESHEET           |
    |   +----------------+   |
    |   |                |   |
    |   +----------------+   |
    |                        |
    |  </head>               |
    |                        |
    |  <body>                |
    |                        |
    |   HEADING              |
    |   +----------------+   |
    |   |                |   |
    |   +----------------+   |
    |                        |
    |   REPORT               |
    |   +----------------+   |
    |   |                |   |
    |   +----------------+   |
    |                        |
    |   ENDING               |
    |   +----------------+   |
    |   |                |   |
    |   +----------------+   |
    |                        |
    |  </body>               |
    |</html>                 |
    +------------------------+
    """

    DEFAULT_TITLE = 'RAK Report'
    DEFAULT_DESCRIPTION = ''
    DEFAULT_TESTER='dcpae'

    # ------------------------------------------------------------------------
    # HTML Template

    HTML_TMPL = r"""<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <title>%(title)s</title>
    <meta name="generator" content="%(generator)s"/>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
    <link href="http://libs.baidu.com/bootstrap/3.0.3/css/bootstrap.min.css" rel="stylesheet">
    %(stylesheet)s
</head>
<body >
%(heading)s
%(report)s
%(ending)s

</body>
</html>
"""
    # variables: (title, generator, stylesheet, heading, report, ending)


    # ------------------------------------------------------------------------
    # Stylesheet
    #
    # alternatively use a <link> for external style sheet, e.g.
    #   <link rel="stylesheet" href="$url" type="text/css">

    STYLESHEET_TMPL = """
<style type="text/css" media="screen">
body        { font-family: Microsoft YaHei,Tahoma,arial,helvetica,sans-serif;padding: 20px; font-size: 80%; }
table       { font-size: 100%; }

/* -- heading ---------------------------------------------------------------------- */
.heading {
    margin-top: 0ex;
    margin-bottom: 1ex;
}

.heading .description {
    margin-top: 4ex;
    margin-bottom: 6ex;
}

/* -- report ------------------------------------------------------------------------ */
#total_row  { font-weight: bold; }
.passCase   { color: #5cb85c; }
.failCase   { color: #d9534f; font-weight: bold; }
.errorCase  { color: #f0ad4e; font-weight: bold; }
.hiddenRow  { display: none; }
</style>
"""

    # ------------------------------------------------------------------------
    # Heading
    #

    HEADING_TMPL = """<div class='heading'>
<h1>%(title)s</h1>
%(parameters)s
<p class='description' style="font-size: 14px;">%(description)s</p>
</div>

""" # variables: (title, parameters, description)

    HEADING_ATTRIBUTE_TMPL = """<p class='attribute' style="font-size: 14px;"><strong>%(name)s : </strong> %(value)s</p>
""" # variables: (name, value)


    # ------------------------------------------------------------------------
    # Report
    #
    REPORT_TMPL = """
<p id='show_detail_line'>
<a class="btn btn-primary" >Summary{ %(passrate)s }</a>
<button onclick="btnFailed_Click()" class="btn btn-danger" >Failed{ %(fail)s }</button>
<button onclick="btnPassed_Click()" class="btn btn-success" >Passed{ %(Pass)s }</button>
<button onclick="btnAll_Click()" class="btn btn-info" >ALL{ %(count)s }</button>
</p>
<table id='result_table' class="table table-condensed table-bordered table-hover">
<colgroup>
<col align='left' />
<col align='right' />
<col align='right' />
<col align='right' />
</colgroup>
<tr id='header_row' class="text-center success" style="font-weight: bold;font-size: 14px;">
    <td width= "150">Automation Level</td>
    <td width= "200">Date</td>
    <td width= "150">Running Time</td>
    <td width= "200">Case_name</td>
    <td width= "100">Result</td>
    <td>Details</td>
    <td>Log</td>
</tr>
%(test_list)s
</table>
""" # variables: (test_list, count, Pass, fail, error ,passrate)

    # ------------------------------------------------------------------------
    # failed cases with log output
    REPORT_FAILED_CASE_TMPL = r"""
<tr class="text-center" style="font-size: 14px;">
    <td width= "150"><div>%(automation_level)s</div></td>
    <td width= "200"><div>%(date)s</div></td>
    <td width= "150"><div>%(runtime)s</div></td>
    <td width= "200"><div>%(case_name)s</div></td>
    <td class='%(style)s' width= "100"><div>%(result)s</div></td>
    <td class='%(style)s'><div>%(details)s</div></td>
    <td colspan='5' align='center'>
        %(serial_log_output)s 
        %(bmc_log_output)s 
        %(os_log_output)s 
        %(cscripts_log_output)s
        %(sel_log_output)s
    </td>
</tr>

<script type="text/javascript">
    function btnFailed_Click(){
        var tableId = document.getElementById('result_table');
        var rowsLength = tableId.rows.length;
        for(var i=1;i<rowsLength;i++){
            var searchText = tableId.rows[i].cells[4].innerText;//取得table行，列的值
            console.log(searchText)
            if(searchText == "failed"){//用match函数进行筛选，如果input的值，即变量 key的值为空，返回的是ture，
                tableId.rows[i].style.display='';//显示行操作，
            }else{
                tableId.rows[i].style.display='none';//隐藏行操作
            }
        }
    }
    
    function btnPassed_Click(){
        var tableId = document.getElementById('result_table');
        var rowsLength = tableId.rows.length;
        for(var i=1;i<rowsLength;i++){
            var searchText = tableId.rows[i].cells[4].innerText;//取得table行，列的值
            console.log(searchText)
            if(searchText == "pass"){//用match函数进行筛选，如果input的值，即变量 key的值为空，返回的是ture，
                tableId.rows[i].style.display='';//显示行操作，
            }else{
                tableId.rows[i].style.display='none';//隐藏行操作
            }
        }
    }
    
    function btnAll_Click(){
        var tableId = document.getElementById('result_table');
        var rowsLength = tableId.rows.length;
        for(var i=1;i<rowsLength;i++){
            tableId.rows[i].style.display='';//显示行操作，
        }
    }
</script>
""" # variables: (style, date, case_name, result, details, log_url)

    # ------------------------------------------------------------------------
    #passed cases
    REPORT_PASSED_CASE_TMPL = r"""
<tr class="text-center" style="font-size: 14px;">
    <td width= "150"><div>%(automation_level)s</div></td>
    <td width= "200"><div>%(date)s</div></td>
    <td width= "150"><div>%(runtime)s</div></td>
    <td width= "200"><div>%(case_name)s</div></td>
    <td class='%(style)s' width= "100"><div>%(result)s</div></td>
    <td class='%(style)s'><div>%(details)s</div></td>
    <td colspan='5' align='center'>
        %(serial_log_output)s 
        %(bmc_log_output)s 
        %(os_log_output)s 
        %(cscripts_log_output)s
        %(sel_log_output)s
    </td>
</tr>
""" # variables: ( style, date, case_name, result, details, serial_log_output, bmc_log_output)

    REPORT_TEST_OUTPUT_TMPL = r"""
%(id)s: %(output)s
""" # variables: (id, output)

    # ------------------------------------------------------------------------
    # ENDING
    #
    ENDING_TMPL = """<div id='ending'>&nbsp;</div>
    <div style=" position:fixed;right:50px; bottom:30px; width:20px; height:20px;cursor:pointer">
    <a href="#"><span class="glyphicon glyphicon-eject" style = "font-size:30px;" aria-hidden="true">
    </span></a></div>
    """

# -------------------- The end of the Template class -------------------

# class CaseResult:
#     def __init__(self, time, case, res, errorMsg, serialLog, bmcLog, osLog, cscriptLog):
#         self.time = time
#         self.case = case
#         self.res = res
#         self.errorMsg = errorMsg
#         self.serialLog = serialLog
#         self.bmcLog = bmcLog
#         self.osLog = osLog
#         self.cscriptLog = cscriptLog

#refactor the format of result（if necessary）
class Result():
    def __init__(self, result):
        self.success_count = 0
        self.failure_count = 0
        self.error_count = 0
        self.cases = []
        self.passrate = 0
        try:
            for case in result:
                self.cases.append(case)
                if "pass" in case:
                    self.success_count += 1
                elif "failed" in case:
                    self.failure_count += 1
                else:
                    self.error_count += 1
            self.passrate = str("%.2f%%" % (float(self.success_count) / float(
                    self.success_count + self.failure_count + self.error_count) * 100))
        except Exception as e:
            pass

class HTMLReport_Generator(Template_HTML_Report):
    """
    """
    def __init__(self, stream=sys.stdout, title=None, description=None, tester=None):
        self.stream = stream
        if title is None:
            self.title = self.DEFAULT_TITLE
        else:
            self.title = title
        if description is None:
            self.description = self.DEFAULT_DESCRIPTION
        else:
            self.description = description
        if tester is None:
            self.tester = self.DEFAULT_TESTER
        else:
            self.tester = tester

        self.startTime = datetime.datetime.now()


    def report(self, result):
        self.generateReport(result)
        print('\nGenerate HTML Report Success' , file=sys.stderr)
        return

    def getReportAttributes(self, result):
        """
        Override this to add custom attributes.
        """
        status = []
        status.append('ALL %s' % (result.success_count + result.failure_count + result.error_count))
        if result.success_count: status.append('Pass %s' % result.success_count)
        if result.failure_count: status.append('Failure %s' % result.failure_count)
        if result.error_count:   status.append('Error %s' % result.error_count)
        if status:
            status = ' '.join(status)
            self.passrate = str("%.2f%%" % (float(result.success_count) / float(
                result.success_count + result.failure_count + result.error_count) * 100))
        else:
            status = 'none'
        return [
            ('Tester', self.tester),
            ('Result', status + ", Passing rate: " + result.passrate),
        ]

    def generateReport(self, result):
        report_attrs = self.getReportAttributes(result)
        generator = 'RAK_HTML_Report %s' % __version__
        stylesheet = self._generate_stylesheet()
        heading = self._generate_heading(report_attrs)
        report = self._generate_report(result)
        ending = self._generate_ending()
        output = self.HTML_TMPL % dict(
            title=saxutils.escape(self.title),
            generator=generator,
            stylesheet=stylesheet,
            heading=heading,
            report=report,
            ending=ending,
        )
        self.stream.write(output.encode('utf8'))

    def _generate_stylesheet(self):
        return self.STYLESHEET_TMPL

    def _generate_heading(self, report_attrs):
        a_lines = []
        for name, value in report_attrs:
            line = self.HEADING_ATTRIBUTE_TMPL % dict(
                name=saxutils.escape(name),
                value=saxutils.escape(value),
            )
            a_lines.append(line)
        heading = self.HEADING_TMPL % dict(
            title=saxutils.escape(self.title),
            parameters=''.join(a_lines),
            description=saxutils.escape(self.description),
            tester=saxutils.escape(self.tester),
        )
        return heading

    def _generate_report(self, result):
        rows = []
        for case in result.cases:
            if "pass" in case:
                tmpl = self.REPORT_PASSED_CASE_TMPL
            else:
                tmpl = self.REPORT_FAILED_CASE_TMPL

            serial_log_output = ""
            bmc_log_output= ""
            os_log_output = ""
            cscripts_log_output = ""
            sel_log_output = ""
            if case[6] == "None":
                serial_log_output = " serial_log "
            else:
                serial_log_output = "<a href="+case[6]+">serial_log&nbsp&nbsp</a>"        #add serial log link
            if case[7] == "None":
                bmc_log_output = " bmc_log "
            else:
                bmc_log_output = "<a href="+case[7]+">bmc_log&nbsp&nbsp</a>"                        #add bmc log link
            if case[8] == "None":
                os_log_output = " os_log "
            else:
                os_log_output = "<a href="+case[8]+">os_log&nbsp&nbsp</a>"
            if case[9] == "None":
                cscripts_log_output = " cscripts_log "
            else:
                cscripts_log_output = "<a href="+case[9]+">cscripts_log</a>"
            if case[10] == "None":
                sel_log_output = " sel_log "
            else:
                sel_log_output = "<a href="+case[10]+">sel_log</a>"

            row = tmpl % dict(
                style = "passCase" if "pass" in case else "failCase",
                automation_level = case[0],
                date = case[1],
                runtime = case[2],
                case_name = case[3],
                result = case[4],
                details = case[5],
                serial_log_output = serial_log_output,
                bmc_log_output = bmc_log_output,
                os_log_output = os_log_output,
                cscripts_log_output = cscripts_log_output,
                sel_log_output = sel_log_output,
            )
            rows.append(row)

        report = self.REPORT_TMPL % dict(
            test_list=''.join(rows),
            count=str(result.success_count + result.failure_count + result.error_count),
            Pass=str(result.success_count),
            fail=str(result.failure_count),
            error=str(result.error_count),
            passrate=self.passrate,
        )
        return report

    def _generate_ending(self):
        return self.ENDING_TMPL


import os

import wx
from wx import stc
from wx.stc import StyledTextCtrl


class CodeEditor(StyledTextCtrl):
    def __init__(self, parent, style=wx.BORDER_NONE):
        StyledTextCtrl.__init__(self, parent, -1, style=style)
        self.SetUpEditor()
        self.filePath = None

    def SetValue(self, value):
        val = self.GetReadOnly()
        self.SetReadOnly(False)
        self.SetText(value)
        self.EmptyUndoBuffer()
        self.SetSavePoint()
        self.SetReadOnly(val)

    def SetFilePath(self, filePath: str = None):
        if filePath and os.path.isfile(filePath):
            self.filePath = filePath

    def SetEditable(self, val):
        self.SetReadOnly(not val)

    def IsModified(self):
        return self.GetModify()

    def Clear(self):
        self.ClearAll()

    def SetInsertionPoint(self, pos):
        self.SetCurrentPos(pos)
        self.SetAnchor(pos)

    def ShowPosition(self, pos):
        line = self.LineFromPosition(pos)
        self.GotoLine(line)

    def GetLastPosition(self):
        return self.GetLength()

    def GetPositionFromLine(self, line):
        return self.PositionFromLine(line)

    def GetRange(self, start, end):
        return self.GetTextRange(start, end)

    def GetSelection(self):
        return self.GetAnchor(), self.GetCurrentPos()

    def SetSelection(self, start, end):
        self.SetSelectionStart(start)
        self.SetSelectionEnd(end)

    def SelectLine(self, line):
        start = self.PositionFromLine(line)
        end = self.GetLineEndPosition(line)
        self.SetSelection(start, end)

    def SetUpEditor(self):
        import keyword

        self.SetLexer(stc.STC_LEX_PYTHON)
        self.SetKeyWords(0, " ".join(keyword.kwlist))

        self.SetProperty("fold", "1")

        self.SetProperty("tab.timmy.whinge.level", "1")

        self.SetMargins(2, 2)

        self.SetMarginType(1, wx.stc.STC_MARGIN_NUMBER)
        self.SetMarginWidth(1, 40)

        self.SetIndent(4)  # Proscribed indent size for wx
        self.SetIndentationGuides(True)  # Show indent guides
        self.SetBackSpaceUnIndents(True)  # Backspace unindents rather than delete 1 space
        self.SetTabIndents(True)  # Tab key indents
        self.SetTabWidth(4)  # Proscribed tab size for wx
        self.SetUseTabs(False)  # Use spaces rather than tabs, or
        self.SetViewWhiteSpace(False)  # Don't view white space
        self.SetEOLMode(wx.stc.STC_EOL_LF)
        self.SetViewEOL(False)
        self.SetEdgeMode(stc.STC_EDGE_NONE)
        self.SetMarginType(2, stc.STC_MARGIN_SYMBOL)
        self.SetMarginMask(2, stc.STC_MASK_FOLDERS)
        self.SetMarginSensitive(2, True)
        self.SetMarginWidth(2, 12)

        self.MarkerDefine(stc.STC_MARKNUM_FOLDEREND, stc.STC_MARK_BOXPLUSCONNECTED, "white",
                          "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDEROPENMID, stc.STC_MARK_BOXMINUSCONNECTED,
                          "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDERMIDTAIL, stc.STC_MARK_TCORNER, "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDERTAIL, stc.STC_MARK_LCORNER, "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDERSUB, stc.STC_MARK_VLINE, "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDER, stc.STC_MARK_BOXPLUS, "white", "black")
        self.MarkerDefine(stc.STC_MARKNUM_FOLDEROPEN, stc.STC_MARK_BOXMINUS, "white", "black")

        if wx.Platform == '__WXMSW__':
            self.StyleSetSpec(stc.STC_STYLE_DEFAULT,
                              'fore:#000000,back:#FFFFFF,face:Courier New')
        else:
            defsize = wx.SystemSettings.GetFont(wx.SYS_ANSI_FIXED_FONT).GetPointSize()
            self.StyleSetSpec(stc.STC_STYLE_DEFAULT,
                              'fore:#000000,back:#FFFFFF,face:Courier,size:%d' % defsize)

        self.StyleClearAll()

        self.StyleSetSpec(wx.stc.STC_STYLE_LINENUMBER, 'fore:#000000,back:#99A9C2')
        self.StyleSetSpec(wx.stc.STC_STYLE_BRACELIGHT, 'fore:#00009D,back:#FFFF00')
        self.StyleSetSpec(wx.stc.STC_STYLE_BRACEBAD, 'fore:#00009D,back:#FF0000')
        self.StyleSetSpec(wx.stc.STC_STYLE_INDENTGUIDE, "fore:#CDCDCD")

        self.StyleSetSpec(wx.stc.STC_P_DEFAULT, 'fore:#000000')
        self.StyleSetSpec(wx.stc.STC_P_COMMENTLINE, 'fore:#008000,back:#F0FFF0')
        self.StyleSetSpec(wx.stc.STC_P_COMMENTBLOCK, 'fore:#008000,back:#F0FFF0')
        self.StyleSetSpec(wx.stc.STC_P_NUMBER, 'fore:#008080')
        self.StyleSetSpec(wx.stc.STC_P_STRING, 'fore:#800080')
        self.StyleSetSpec(wx.stc.STC_P_CHARACTER, 'fore:#800080')
        self.StyleSetSpec(wx.stc.STC_P_WORD, 'fore:#000080,bold')
        self.StyleSetSpec(wx.stc.STC_P_TRIPLE, 'fore:#800080,back:#FFFFEA')
        self.StyleSetSpec(wx.stc.STC_P_TRIPLEDOUBLE, 'fore:#800080,back:#FFFFEA')
        self.StyleSetSpec(wx.stc.STC_P_CLASSNAME, 'fore:#0000FF,bold')
        self.StyleSetSpec(wx.stc.STC_P_DEFNAME, 'fore:#008080,bold')
        self.StyleSetSpec(wx.stc.STC_P_OPERATOR, 'fore:#800000,bold')
        self.StyleSetSpec(wx.stc.STC_P_IDENTIFIER, 'fore:#000000')

        self.SetCaretForeground("BLUE")
        self.SetSelBackground(1, '#66CCFF')

        self.SetSelBackground(True, wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHT))
        self.SetSelForeground(True, wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT))

    def RegisterModifiedEvent(self, eventHandler):
        self.Bind(wx.stc.EVT_STC_CHANGE, eventHandler)



import re

import setuptools

with open("rak_tools/__init__.py", "r") as file:
    regex_version = r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]'
    version = re.search(regex_version, file.read(), re.MULTILINE).group(1)

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rak_tools",  # Replace with your own username
    version=version,
    author="marvinYu",
    author_email="marvin_ywf@163.com",
    description="UI package for lightras tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "wxPython>=4.1.0",
    ],
    python_requires='>=3.6',
)
